import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { CommandHandler } from '@nestjs/cqrs';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { CreateTerritoryCommand } from '../../commands/create-territory.command';
import { TerritoryMessager } from '../../adapters/secondaries/territory.messager';
import { Territory } from '../entities/territory';
import { CreateTerritoryRequest } from '../dtos/create-territory.request';
import { TerritoryLoggingPresenter } from '../../adapters/secondaries/territory-logging.presenter';

@CommandHandler(CreateTerritoryCommand)
export class CreateTerritoryUseCase {
  constructor(
    private readonly _repository: TerritoriesRepository,
    private readonly _territoryMessager: TerritoryMessager,
    private readonly _loggingMessager: LoggingMessager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  async execute(command: CreateTerritoryCommand): Promise<Territory> {
    const entity = this._mapper.map(
      command.createTerritoryRequest,
      CreateTerritoryRequest,
      Territory,
    );

    try {
      const territory = await this._repository.createTerritory(entity);
      this._territoryMessager.publish(
        'create',
        JSON.stringify(
          this._mapper.map(territory, Territory, TerritoryLoggingPresenter),
        ),
      );
      this._loggingMessager.publish(
        'territory.create.info',
        JSON.stringify(
          this._mapper.map(territory, Territory, TerritoryLoggingPresenter),
        ),
      );
      return territory;
    } catch (error) {
      let key = 'territory.create.crit';
      if (error.message.includes('already exists')) {
        key = 'territory.create.warning';
      }
      this._loggingMessager.publish(
        key,
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
