import { CommandHandler } from '@nestjs/cqrs';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { TerritoryMessager } from '../../adapters/secondaries/territory.messager';
import { DeleteTerritoryCommand } from '../../commands/delete-territory.command';
import { Territory } from '../entities/territory';

@CommandHandler(DeleteTerritoryCommand)
export class DeleteTerritoryUseCase {
  constructor(
    private readonly _repository: TerritoriesRepository,
    private readonly _territoryMessager: TerritoryMessager,
    private readonly _loggingMessager: LoggingMessager,
  ) {}

  async execute(command: DeleteTerritoryCommand): Promise<Territory> {
    try {
      const territory = await this._repository.delete(command.uuid);
      this._territoryMessager.publish(
        'delete',
        JSON.stringify({ uuid: territory.uuid }),
      );
      this._loggingMessager.publish(
        'territory.delete.info',
        JSON.stringify({ uuid: territory.uuid }),
      );
      return territory;
    } catch (error) {
      this._loggingMessager.publish(
        'territory.delete.crit',
        JSON.stringify({
          command,
          error,
        }),
      );
      throw error;
    }
  }
}
