import { classes } from '@automapper/classes';
import { AutomapperModule } from '@automapper/nestjs';
import { Test, TestingModule } from '@nestjs/testing';
import { LoggingMessager } from '../../adapters/secondaries/logging.messager';
import { TerritoriesRepository } from '../../adapters/secondaries/territories.repository';
import { TerritoryMessager } from '../../adapters/secondaries/territory.messager';
import { UpdateTerritoryCommand } from '../../commands/update-territory.command';
import { UpdateTerritoryRequest } from '../../domain/dtos/update-territory.request';
import { Territory } from '../../domain/entities/territory';
import { UpdateTerritoryUseCase } from '../../domain/usecases/update-territory.usecase';
import { TerritoryProfile } from '../../mappers/territory.profile';

const originalTerritory: Territory = new Territory();
originalTerritory.uuid = 'bb281075-1b98-4456-89d6-c643d3044a91';
originalTerritory.name = 'Grand Est';

const updateTerritoryRequest: UpdateTerritoryRequest = {
  uuid: 'bb281075-1b98-4456-89d6-c643d3044a91',
  name: 'Grand-Est',
};

const updateTerritoryCommand: UpdateTerritoryCommand =
  new UpdateTerritoryCommand(updateTerritoryRequest);

const mockTerritoriesRepository = {
  updateTerritory: jest
    .fn()
    .mockImplementationOnce((uuid: string, params: any) => {
      originalTerritory.name = params.name;

      return Promise.resolve(originalTerritory);
    })
    .mockImplementation(() => {
      throw new Error('Error');
    }),
};

const mockMessager = {
  publish: jest.fn().mockImplementation(),
};

describe('UpdateTerritoryUseCase', () => {
  let updateTerritoryUseCase: UpdateTerritoryUseCase;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [AutomapperModule.forRoot({ strategyInitializer: classes() })],

      providers: [
        {
          provide: TerritoriesRepository,
          useValue: mockTerritoriesRepository,
        },
        UpdateTerritoryUseCase,
        TerritoryProfile,
        {
          provide: TerritoryMessager,
          useValue: mockMessager,
        },
        {
          provide: LoggingMessager,
          useValue: mockMessager,
        },
      ],
    }).compile();

    updateTerritoryUseCase = module.get<UpdateTerritoryUseCase>(
      UpdateTerritoryUseCase,
    );
  });

  it('should be defined', () => {
    expect(updateTerritoryUseCase).toBeDefined();
  });

  describe('execute', () => {
    it('should update a territory', async () => {
      const updatedTerritory: Territory = await updateTerritoryUseCase.execute(
        updateTerritoryCommand,
      );

      expect(updatedTerritory.name).toBe(updateTerritoryRequest.name);
    });
    it('should throw an error if territory does not exist', async () => {
      await expect(
        updateTerritoryUseCase.execute(updateTerritoryCommand),
      ).rejects.toBeInstanceOf(Error);
    });
  });
});
